<?php

declare(strict_types=1);

namespace Drupal\paragraph_lineage;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Theme\ThemeManagerInterface;

/**
 * Service for managing paragraph lineage settings.
 */
final class ParagraphLineageSettingsService implements ParagraphLineageSettingsServiceInterface {

  const string CONFIG_NAME = 'paragraph_lineage.settings';

  /**
   * Constructs a ParagraphLineageSettingsService.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typedConfigManager
   *   The typed config manager.
   * @param \Drupal\Core\Theme\ThemeManagerInterface $themeManager
   *   The theme manager.
   */
  public function __construct(
    protected ConfigFactoryInterface $configFactory,
    protected TypedConfigManagerInterface $typedConfigManager,
    protected ThemeManagerInterface $themeManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function getPreferredThemeId(): string {
    // Retrieve the preferred theme from the configuration.
    $config = $this->configFactory->get(self::CONFIG_NAME);
    $theme = $config->get('preferred_theme');

    // If no theme is set, return the admin theme.
    if (!$theme) {
      // Get the admin theme from the typed config manager.
      $theme = 'admin';
    }

    return $theme;
  }

  /**
   * {@inheritdoc}
   */
  public function setPreferredThemeId(string $theme): void {
    // Set the preferred theme in the configuration.
    $config = $this->configFactory->getEditable(self::CONFIG_NAME);
    $config->set('preferred_theme', $theme)->save();
  }

}
